@@ -36,8 +36,11 @@ struct Task<T> {
 /// Implement what we would like to do when a task gets woken up
 impl<T> Woke for Task<T> {
     fn wake_by_ref(_: &Arc<Self>) {
-        // poll everything because future is done and may have created conditions for something to finish
-        DEFAULT_EXECUTOR.lock().poll_tasks()
+        // we check if there's a lock because some immediate executing futures screw this up
+        if let Some(mut e) = DEFAULT_EXECUTOR.try_lock() {
+            // poll everything because future is done and may have created conditions for something to finish
+            e.poll_tasks()
+        }
     }
 }
 
@@ -54,7 +57,7 @@ impl<T> Pendable for Arc<Task<T>> {
 
 impl Executor {
     // Run async task
-    pub fn run<T>(&mut self, future: Box<dyn Future<Output = T> + 'static + Send + Unpin>)
+    pub fn run<T>(&mut self, future: Pin<Box<dyn Future<Output = T> + 'static + Send>>)
     where
         T: Send + 'static,
     {
@@ -65,17 +68,16 @@ impl Executor {
     /// Add task for a future to the list of tasks
     fn add_task<T>(
         &mut self,
-        future: Box<dyn Future<Output = T> + 'static + Send + Unpin>,
-    ) -> Arc<Task<T>>
+        future: Pin<Box<dyn Future<Output = T> + 'static + Send>>,
+    )
     where
         T: Send + 'static,
     {
         // store our task
         let task = Arc::new(Task {
-            future: Mutex::new(Box::pin(future)),
+            future: Mutex::new(future),
         });
-        self.tasks.push_back(Box::new(task.clone()));
-        task
+        self.tasks.push_back(Box::new(task));
     }
 
     // Poll all tasks on global executor
@@ -90,12 +92,12 @@ impl Executor {
 }
 
 lazy_static! {
-    static ref DEFAULT_EXECUTOR: Mutex<Box<Executor>> = Mutex::new(Box::new(Executor::default()));
+    static ref DEFAULT_EXECUTOR: Mutex<Executor> = Mutex::new(Executor::default());
 }
 
 pub fn run<T>(future: impl Future<Output = T> + 'static + Send)
 where
     T: Send + 'static,
 {
-    DEFAULT_EXECUTOR.lock().run(Box::new(Box::pin(future)))
+    DEFAULT_EXECUTOR.lock().run(Box::pin(future))
 }
